#!/usr/bin/env python

import os
import pygame
from pygame.locals import *
from pygame.compat import geterror
from random import randint

# import the scenario file
from scenario_data import *

main_dir = os.path.split(os.path.abspath(__file__))[0]
data_dir = os.path.join(main_dir, 'data')

def load_image(name, colorkey=None):
    fullname = os.path.join(data_dir, name)
    try:
        image = pygame.image.load(fullname)
    except pygame.error:
        print ('Cannot load image:', fullname)
        raise SystemExit(str(geterror()))
    image = image.convert()
    if colorkey is not None:
        if colorkey is -1:
            colorkey = image.get_at((0, 0))
        image.set_colorkey(colorkey, RLEACCEL)
    return image, image.get_rect()

def load_sound(name):
    fullname = os.path.join(data_dir, name)
    try:
        sound = pygame.mixer.Sound(fullname)
    except pygame.error:
        print ('Cannot load sound: %s' % fullname)
        raise SystemExit(str(geterror()))
    return sound

def load_music(name):
    fullname = os.path.join(data_dir, name)
    try:
        pygame.mixer.music.load(fullname)
    except pygame.error:
        print ('Cannot load music: %s' % fullname)
        raise SystemExit(str(geterror()))

def load_font(name):
    fullname = os.path.join(data_dir, name)
    try:
        result = pygame.font.Font(fullname,  10)
    except pygame.error:
        result = pygame.font.Font(None, 10)
        print ('Cannot load font:', fullname)
        raise SystemExit(str(geterror()))
    return result

# hand-shaped cursor
class Hand(pygame.sprite.DirtySprite):
    def __init__(self):
        pygame.sprite.DirtySprite.__init__(self)
        self.image, self.rect = load_image('hand_cursor.bmp', -1)
        self.selecting = 0
        self.timer = 0

    def move(self):
        pos = pygame.mouse.get_pos()
        self.rect.midtop = pos
        self.dirty = 1

    def update(self):
        self.dirty = 1

    def hit_check(self, target_list):
        if not self.selecting:
            self.selecting = 1
            self.timer = 20
            hitbox = self.rect.inflate(-5, -5)
            return hitbox.collidelist(target_list)

    def holding(self):
        if self.timer > 0:
            self.timer -= 1
        else:
            self.selecting = 0
            self.dirty = 1

class Balloon(pygame.sprite.DirtySprite):
    def __init__(self, (pos_x, pos_y),  (width, height)):
        pygame.sprite.DirtySprite.__init__(self)
        self.image = pygame.Surface((width, height))
        self.image = self.image.convert()
        self.image.fill((255, 255, 255))
        self.original = self.image
        self.rect = self.image.get_rect()
        
        self.rect.topleft = pos_x, pos_y
        self.anim_diff = 1
        self.anim_cnt = randint(0,10)

    def update(self):
        self.anim_cnt = self.anim_cnt + self.anim_diff
        # inflating animation
        center = self.rect.center
        if self.anim_cnt >= 100:
            self.anim_diff = -1
        elif self.anim_cnt <= 0:
            self.anim_diff = 1
            #self.image = self.original
        scaling = pygame.transform.scale
        if self.anim_cnt % 10 == 0:
            self.anim_rect = self.rect.inflate(self.anim_diff, self.anim_diff)
            self.image = scaling(self.image, (self.anim_rect[2],  self.anim_rect[3]))
            self.rect = self.image.get_rect(center=center)
        
        self.dirty = 1

class Small_Balloon(pygame.sprite.DirtySprite):
    def __init__(self, (pos_x, pos_y),  (width,  height),  (dx,  dy)):
        pygame.sprite.DirtySprite.__init__(self)
        self.image = pygame.Surface((width, height))
        self.image = self.image.convert()
        self.image.fill((255, 255, 255))
        self.original = self.image
        self.rect = self.image.get_rect()
        self.original_rect = self.rect
        self.rect.topleft = pos_x, pos_y
        self.anim_diff = 1
        self.anim_cnt = randint(0,10)

        self.move_x = dx
        self.move_y = dy
        self.move_cnt = randint(2,40)

    def show(self):
        self.visible = 1

    def hide(self):
        self.visible = 0

    def update(self):
        self.anim_cnt = self.anim_cnt + self.anim_diff
        # inflating animation
        center = self.rect.center
        if self.anim_cnt >= 40:
            self.anim_diff = -1
        elif self.anim_cnt <= 0:
            self.anim_diff = 1
            #self.image = self.original
        scaling = pygame.transform.scale
        if self.anim_cnt % 10 == 0:
            self.anim_rect = self.rect.inflate(self.anim_diff, self.anim_diff)
            self.image = scaling(self.image, (self.anim_rect[2],  self.anim_rect[3]))
            self.rect = self.image.get_rect(center=center)

        self.move_cnt += 1
        if self.move_cnt >= 100:
            self.move_cnt = randint(2,40)
            self.rect = self.original_rect
        elif self.move_cnt % 10 == 0:
            newpos = self.rect.move((self.move_x, self.move_y))
            self.rect = newpos        

        self.dirty = 1

class Blood_Particle(pygame.sprite.DirtySprite):
    def __init__(self):
        pygame.sprite.DirtySprite.__init__(self)
        self.image = pygame.Surface((randint(5, 9), randint(5, 9)))
        self.image = self.image.convert()
        self.image.fill((255, 0, 0))
        self.rect = self.image.get_rect()
        self.original_rect = self.rect

        self.move_x = 0
        self.move_y = 0
        self.move_cnt = 0
        self.splash_flag = 0
        self.visible = 0

    def show(self):
        self.visible = 1

    def hide(self):
        self.visible = 0

    def splash_start(self,  (x, y)):
        self.splash_flag = 1
        self.rect.topleft = x, y
        self.move_cnt = randint(20,40)
        self.move_x = randint(0, 8)-4
        self.move_y = randint(0, 8)-8
        self.visible = 1
                
    def update(self):
        if self.splash_flag == 1:
            self.move_cnt -= 1
            if self.move_cnt < 0:
                self.splash_flag == 0
                self.visible = 0
            elif self.move_cnt % 2 == 0:
                self.move_y = self.move_y + 1
                newpos = self.rect.move((self.move_x, self.move_y))
                self.rect = newpos        

        self.dirty = 1

class Words(pygame.sprite.DirtySprite):
    def __init__(self):
        pygame.sprite.DirtySprite.__init__(self)

    def show(self, (pos_x, pos_y),  lines,  game_font):
        if type(lines) is str:
            lines = (lines, )
        size_check = game_font.render((max(lines, key=len)), 1, (0, 0, 0))
        size_line = size_check.get_height()
        size_x = size_check.get_width()
        size_y = len(lines) * (size_line + 12)
        self.image = pygame.Surface((size_x, size_y))
        self.image.set_colorkey((255, 255, 255))
        self.image = self.image.convert()
        self.image.fill((255, 255, 255))

        for i, line in enumerate(lines):
            text = game_font.render(line, 1, (0, 0, 0))
            self.image.blit(text, (0, (i * (size_line + 8))))

        self.rect = self.image.get_rect()
        self.rect.topleft = pos_x, pos_y

    def update(self):
        self.dirty = 1

class Card_Back(pygame.sprite.DirtySprite):
    def __init__(self, (pos_x, pos_y)):
        pygame.sprite.DirtySprite.__init__(self)

        self.image = pygame.Surface((58, 80))
        self.image.fill((155, 135, 155))

        self.image = self.image.convert()
        self.rect = self.image.get_rect()
        self.rect.topleft = pos_x, pos_y
        self.visible = 0

    def show(self):
        self.visible = 1

    def update(self):
        self.dirty = 1

class Wink(pygame.sprite.DirtySprite):
    def __init__(self, bg_image, (pos_x, pos_y),  (width, height)):
        pygame.sprite.DirtySprite.__init__(self)
        self.image = pygame.Surface((width, height))
        self.image.blit(bg_image, (0, 0),  (pos_x, pos_y,  width, height))     
#        self.image = self.image.convert()
        self.rect = self.image.get_rect()

        self.rect.topleft = pos_x, pos_y
        self.visible = 0
        
    def shut(self):
        self.visible = 1
    def open(self):
        self.visible = 0
        
    def update(self):
        self.dirty = 1

class Scenario():
    def __init__(self,  scenario_data):
        # for scenario control
        self.seq_n = scenario_data
        self.seq_pointer = -1
        self.mes_branch = 0
        self.waiting = 1

    def check(self,  choice):
        if self.waiting == 1 and choice == 0:
            self.waiting = 0
            self.seq_pointer += 1
            return 1
        elif self.waiting == 2 and (choice == 1 or choice == 2):
            self.waiting = 0
            self.mes_branch = choice
            self.seq_pointer += 1
            return 1
        elif self.waiting == 3 and (choice == 1 or choice == 2):
            self.waiting = 0
            ss_n = self.seq_n[self.seq_pointer+1]
            self.seq_n = ss_n[choice-1]
            self.seq_pointer = 0
            return 1
        elif self.waiting == 5 and choice == 0:
            self.waiting = 0
            ss_n = self.seq_n[self.seq_pointer+1]
            self.seq_n = ss_n
            self.seq_pointer = 0
            return 1
        return 0
            
    def next(self):
        if self.waiting != 0:
            return (0, False)

        # whose message?
        if self.mes_branch == 0:
            # normal message
            ss_n = self.seq_n[self.seq_pointer]
        elif self.mes_branch == 1:
            # branched to red's message
            ss_n = self.seq_n[self.seq_pointer][0]
        else:
            # branched to blue's message
            ss_n = self.seq_n[self.seq_pointer][1]

        self.mes_branch = 0
        self.seq_pointer += 1
        if self.seq_n[self.seq_pointer] == 0:
            # main message sequence
            self.waiting = 1
            return int(ss_n[4]), ss_n
        elif self.seq_n[self.seq_pointer] == 1:
            # Short Branch Choice
            self.waiting = 2
            return 0, ss_n
        elif self.seq_n[self.seq_pointer] == 2:
            # Route Change Choice
            self.waiting = 3
            return 0, ss_n
        elif self.seq_n[self.seq_pointer] == 3:
            # Game Over
            self.waiting = 1
            return 3, ss_n
        elif self.seq_n[self.seq_pointer] == 4:
            # Event
            self.waiting = 1
            return 4, ss_n
        elif self.seq_n[self.seq_pointer] == 5:
            # Route Change
            self.waiting = 5
            return 0, ss_n

def main():
    pygame.init()
    screen = pygame.display.set_mode((500, 500))
    pygame.display.set_caption('PyWeek#23 Challenge')
    pygame.mouse.set_visible(0)

    background = pygame.Surface(screen.get_size())
    (bg_image, rect)  = load_image('background.png')
    (bg_anim, rect)  = load_image('background_anim.png')

    background.fill((0, 0, 0))
    background.blit(bg_image, (0,0))

    background = background.convert()

    clock = pygame.time.Clock()

    cancel_sound = load_sound('cancel2.wav')
    select_sound = load_sound('select5.wav')
    select_sound.set_volume(0.8)
    death_sound = load_sound('Sheep Bleating.wav')
    kill_sound = load_sound('death.wav')
    kill_sound.set_volume(0.5)

    load_music('Pocketmaster_-_07_-_Ride.mp3')
    
    game_font = load_font('FGKlara.ttf')

    words_main = Words()
    words_main.show((157, 247),  ("", "      The Human Hospital", "    For Pyweek#23 Challenge"),  game_font)
    words_left = Words()
    words_left.show((135, 147),  ("Hi.", ),  game_font)
    words_right = Words()
    words_right.show((271,  143),  ("Hello.", ),  game_font)
    words_top = Words()
    words_top.show((179,  46),  ("Click the main dialog.", ),  game_font)

    balloon_left = Balloon((125, 137), (101, 41))
    balloon_right = Balloon((261, 133), (103, 46))
    balloon_main = Balloon((147, 237), (221, 102))
    balloon_top = Balloon((169, 36), (167, 45))

    small_balloon_obj = []
    small_balloon_obj.append(Small_Balloon((130, 239), (17, 17), (-1.5, -1)))
    small_balloon_obj.append(Small_Balloon((130-15, 239-20), (7, 7), (-1.2, -1)))
    small_balloon_obj.append(Small_Balloon((368, 237), (17, 17), (1.5, -1)))
    small_balloon_obj.append(Small_Balloon((368+15, 237-20), (7, 7), (1.2, -1)))
    
    small_balloon_obj[0].hide()
    small_balloon_obj[1].hide()
    small_balloon_obj[2].hide()
    small_balloon_obj[3].hide()

    blood_particle_obj = []
    for i in xrange(0, 8):
        blood_particle_obj.append(Blood_Particle())

    hand_cursor = Hand()

    eye_left = Wink(bg_anim, (69, 132), (50, 30))
    eye_right = Wink(bg_anim, (362, 130), (54, 29))
    
    card_obj = []
    card_obj.append(Card_Back((80, 396)))
    card_obj.append(Card_Back((179, 391)))
    card_obj.append(Card_Back((277, 392)))
    card_obj.append(Card_Back((370, 399)))
    
    blood_particle_group = [blood_particle_obj[0], blood_particle_obj[1], blood_particle_obj[2], blood_particle_obj[3], blood_particle_obj[4], blood_particle_obj[5], blood_particle_obj[6], blood_particle_obj[7] ]
    small_balloon_group = [small_balloon_obj[0],  small_balloon_obj[1], small_balloon_obj[2],  small_balloon_obj[3]]
    card_group = [card_obj[0],  card_obj[1],  card_obj[2],  card_obj[3]]
    eye_group = [eye_left,  eye_right]
    words_group = [words_main,  words_left,  words_right,  words_top]
    balloon_group = [balloon_left, balloon_right,  balloon_main, balloon_top]
    allsprites = pygame.sprite.LayeredDirty((card_group, eye_group, balloon_group, small_balloon_group,  words_group,  blood_particle_group,  hand_cursor))
    allsprites.clear(screen, background)

    scenario = Scenario(seq1)
    
    blink_left = randint(100,500)
    blink_right = randint(100,500)
    blink_left_hold = 0
    blink_right_hold = 0
    blink_left_flag = 0
    blink_right_flag = 0

    sheep_num = 4
    
    pygame.mixer.music.set_volume(0.2)
    pygame.mixer.music.play(1,  2)

    # main
    going = True
    while going:
        clock.tick(60)
        for event in pygame.event.get():
            if event.type == QUIT:
                going = False
            elif event.type == KEYDOWN and event.key == K_ESCAPE:
                going = False
            elif event.type == MOUSEBUTTONDOWN and hand_cursor.selecting == 0:
                target_index = hand_cursor.hit_check((balloon_main.rect,  balloon_left.rect,  balloon_right.rect))
                if target_index >= 0 or target_index <= 2:
                    result = scenario.check(target_index)
                    if result == 1:
                        select_sound.play()
                        (event_n, ss_n) = scenario.next()
                        rotate = pygame.transform.rotate
                        #print event_n
                        for i in xrange(0, 4):
                            small_balloon_obj[i].hide()
                        # show messages
                        if ss_n[0] != ():
                            r1 = randint(0,5)-3
                            r2 = randint(0,5)-3
                            words_top.show((179+r1,  46+r2),  ss_n[0],  game_font)
                            words_top.image = rotate(words_top.image, r1/2)
                            words_top.visible = 1
                            balloon_top.visible = 1
                        else:
                            words_top.visible = 0
                            balloon_top.visible = 0
                        if ss_n[1] != ():
                            r1 = randint(0,5)-3
                            r2 = randint(0,5)-3
                            words_left.show((135+r1, 147+r2),  ss_n[1],  game_font)
                            words_left.image = rotate(words_left.image, r2/2)
                            words_left.visible = 1
                            balloon_left.visible = 1
                        else:
                            words_left.visible = 0
                            balloon_left.visible = 0
                        if ss_n[2] != ():
                            r1 = randint(0,5)-3
                            r2 = randint(0,5)-3
                            words_right.show((271+r1,  143+r2),  ss_n[2],  game_font)
                            words_right.image = rotate(words_right.image, r1/2)
                            words_right.visible = 1
                            balloon_right.visible = 1
                        else:
                            words_right.visible = 0
                            balloon_right.visible = 0
                        if ss_n[3] != ():
                            r1 = randint(0,5)-3
                            r2 = randint(0,5)-3
                            words_main.show((157+r1, 247+r2),  ss_n[3],  game_font)
                            #words_main.image = rotate(words_main.image, r2/2)
                            words_main.visible = 1
                            balloon_main.visible = 1
                            # small balloon control
                            if event_n == 0:
                                for i in xrange(0, 4):
                                    small_balloon_obj[i].hide()
                            if event_n == 1:
                                small_balloon_obj[0].show()
                                small_balloon_obj[1].show()
                                small_balloon_obj[2].hide()
                                small_balloon_obj[3].hide()                                
                            if event_n == 2:
                                small_balloon_obj[0].hide()
                                small_balloon_obj[1].hide()
                                small_balloon_obj[2].show()
                                small_balloon_obj[3].show()                                
                        else:
                            words_main.visible = 0
                            balloon_main.visible = 0

                        if event_n == 3:
                            # game end
                            print ("game over!")
                            going = False
                        if event_n == 4:
                            # killing
                            card_obj[sheep_num-1].show()
                            death_sound.play()
                            kill_sound.play()

                            x = ((80+24, 396+40), (179+24, 391+40), (277+24, 392+40), (370+24, 399+40) )
                            for i in xrange(0, 8):
                                blood_particle_obj[i].splash_start(x[sheep_num-1])
                            sheep_num -= 1
                            if sheep_num == 0:
                                print ("game over!")
                                going = False
                    else:
                        cancel_sound.play()
                else:
                    cancel_sound.play()
            elif event.type == MOUSEMOTION:
                hand_cursor.move()
        if hand_cursor.selecting == 1:
            hand_cursor.holding()

        # blinking
        if blink_left_hold > 0:
            blink_left_hold -= 1
        if blink_left_hold == 0 and blink_left_flag == 1:
            eye_left.open()
            blink_left_flag = 0
        if blink_left_flag == 0:
            if blink_left > 0:
                blink_left -= 1
            else:
                blink_left = randint(100,500)
                eye_left.shut()
                blink_left_hold = randint(3,6)
                blink_left_flag = 1

        if blink_right_hold > 0:
            blink_right_hold -= 1
        if blink_right_hold == 0 and blink_right_flag == 1:
            eye_right.open()
            blink_right_flag = 0
        if blink_left_flag == 0:
            if blink_right > 0:
               blink_right -= 1
            else:
                blink_right = randint(100,500)
                eye_right.shut()
                blink_right_hold = randint(7,12)
                blink_right_flag = 1

        allsprites.update()
        allsprites.clear(screen, background)

        rects = allsprites.draw(screen)
        pygame.display.update(rects)

    pygame.quit()

if __name__ == '__main__':
    main()
